@@ -3,16 +3,19 @@ import six
 import time
 import numpy as np
 import pandas as pd
+from pathlib import Path
+from warnings import warn
 from dexom_python.imat_functions import imat
 from dexom_python.result_functions import read_solution, write_solution
-from dexom_python.model_functions import load_reaction_weights, read_model, check_model_options
+from dexom_python.model_functions import load_reaction_weights, read_model, check_model_options, DEFAULT_VALUES
 from dexom_python.enum_functions.enumeration import EnumSolution, get_recent_solution_and_iteration, create_enum_variables
 from dexom_python.enum_functions.icut_functions import create_icut_constraint
 from dexom_python.enum_functions.maxdist_functions import create_maxdist_constraint, create_maxdist_objective
 
 
-def diversity_enum(model, reaction_weights, prev_sol=None, eps=1e-3, thr=1e-5, obj_tol=1e-3, maxiter=10, dist_anneal=0.995,
-                   out_path='enum_dexom', icut=True, full=False, save=False):
+def diversity_enum(model, reaction_weights, prev_sol=None, eps=DEFAULT_VALUES['epsilon'], thr=DEFAULT_VALUES['threshold'],
+                   obj_tol=DEFAULT_VALUES['obj_tol'], maxiter=DEFAULT_VALUES['maxiter'],
+                   dist_anneal=DEFAULT_VALUES['dist_anneal'], out_path='enum_dexom', icut=True, full=False, save=False):
     """
     diversity-based enumeration
 
@@ -44,6 +47,7 @@ def diversity_enum(model, reaction_weights, prev_sol=None, eps=1e-3, thr=1e-5, o
     Returns
     -------
     solution: an EnumSolution object
+    stats: a pandas.DataFrame containing the number of selected reactions and runtime of each iteration
     """
     if prev_sol is None:
         prev_sol = imat(model, reaction_weights, epsilon=eps, threshold=thr, full=full)
@@ -98,35 +102,45 @@ def diversity_enum(model, reaction_weights, prev_sol=None, eps=1e-3, thr=1e-5, o
     model.solver.remove([const for const in icut_constraints if const in model.solver.constraints])
     model.solver.remove(opt_const)
     solution = EnumSolution(all_solutions, all_binary, all_solutions[0].objective_value)
-    df = pd.DataFrame({'selected reactions': selected_recs, 'time': times})
+    stats = pd.DataFrame({'selected reactions': selected_recs, 'time': times})
     sol = pd.DataFrame(solution.binary)
     if save:
-        df.to_csv(out_path+time.strftime('%Y%m%d-%H%M%S')+'_results.csv')
+        stats.to_csv(out_path+time.strftime('%Y%m%d-%H%M%S')+'_results.csv')
         sol.to_csv(out_path+time.strftime('%Y%m%d-%H%M%S')+'_solutions.csv')
-    return solution, df
+    return solution, stats
 
 
-if __name__ == '__main__':
+def main():
+    """
+    This function is called when you run this script from the commandline.
+    It performs the reaction-enumeration algorithm on a specified list of reactions
+    Use --help to see commandline parameters
+    """
     description = 'Performs the diversity-enumeration algorithm'
 
     parser = argparse.ArgumentParser(description=description, formatter_class=argparse.RawTextHelpFormatter)
     parser.add_argument('-m', '--model', help='Metabolic model in sbml, matlab, or json format')
     parser.add_argument('-r', '--reaction_weights', default=None,
                         help='Reaction weights in csv format (first row: reaction names, second row: weights)')
-    parser.add_argument('-p', '--prev_sol', default=[], help='starting solution or directory of recent solutions')
-    parser.add_argument('-e', '--epsilon', type=float, default=1e-3,
+    parser.add_argument('-p', '--prev_sol', default=None, help='starting solution or directory of recent solutions')
+    parser.add_argument('-e', '--epsilon', type=float, default=DEFAULT_VALUES['epsilon'],
                         help='Activation threshold for highly expressed reactions')
-    parser.add_argument('--threshold', type=float, default=1e-5, help='Activation threshold for all reactions')
-    parser.add_argument('-t', '--timelimit', type=int, default=None, help='Solver time limit')
-    parser.add_argument('-i', '--maxiter', type=int, default=10, help='Iteration limit')
-    parser.add_argument('--tol', type=float, default=1e-8, help='Solver feasibility tolerance')
-    parser.add_argument('--mipgap', type=float, default=1e-6, help='Solver MIP gap tolerance')
-    parser.add_argument('--obj_tol', type=float, default=1e-2,
+    parser.add_argument('--threshold', type=float, default=DEFAULT_VALUES['threshold'],
+                        help='Activation threshold for all reactions')
+    parser.add_argument('-t', '--timelimit', type=int, default=DEFAULT_VALUES['timelimit'], help='Solver time limit')
+    parser.add_argument('--tol', type=float, default=DEFAULT_VALUES['tolerance'], help='Solver feasibility tolerance')
+    parser.add_argument('--mipgap', type=float, default=DEFAULT_VALUES['mipgap'], help='Solver MIP gap tolerance')
+    parser.add_argument('--obj_tol', type=float, default=DEFAULT_VALUES['obj_tol'],
                         help='objective value tolerance, as a fraction of the original value')
+    parser.add_argument('-i', '--maxiter', type=int, default=DEFAULT_VALUES['maxiter'], help='Iteration limit')
     parser.add_argument('-o', '--output', default='div_enum', help='Base name of output files, without format')
-    parser.add_argument('-a', '--dist_anneal', type=float, default=0.995, help='annealing distance')
-    parser.add_argument('-s', '--startsol_num', type=int, default=1, help='number of starting solutions'
-                                                                          '(if prev_sol is a directory)')
+    parser.add_argument('-a', '--dist_anneal', type=float, default=DEFAULT_VALUES['dist_anneal'],
+                        help='this parameter 0<=a<=1 controls the distance between each successive solution, '
+                             '0 meaning no distance and 1 maximal distance')
+    parser.add_argument('-s', '--startsol', type=int, default=1, help='total number of starting solutions '
+                                                                      '(if prev_sol is a directory)'
+                                                                      'which solution to use as starting point'
+                                                                      '(if prev_sol is a binary solution file)')
     parser.add_argument('--noicut', action='store_true', help='Use this flag to remove the icut constraint')
     parser.add_argument('--full', action='store_true', help='Use this flag to assign non-zero weights to all reactions')
     parser.add_argument('--save', action='store_true', help='Use this flag to save each individual solution')
@@ -138,21 +152,37 @@ if __name__ == '__main__':
     if args.reaction_weights is not None:
         reaction_weights = load_reaction_weights(args.reaction_weights)
     a = args.dist_anneal
-    if '.' in args.prev_sol:
-        prev_sol, prev_bin = read_solution(args.prev_sol, model)
-        model = create_enum_variables(model, reaction_weights, eps=args.epsilon, thr=args.threshold, full=args.full)
-    elif args.prev_sol is not None:
-        prev_sol, i = get_recent_solution_and_iteration(args.prev_sol, args.startsol_num)
-        a = a ** i
-        model = create_enum_variables(model, reaction_weights, eps=args.epsilon, thr=args.threshold, full=args.full)
-    else:
+    prev_sol_success = False
+    if args.prev_sol is not None:
+        prev_sol_path = Path(args.prev_sol)
+        if prev_sol_path.is_file():
+            prev_sol, prev_bin = read_solution(args.prev_sol, model)
+            model = create_enum_variables(model, reaction_weights, eps=args.epsilon, thr=args.threshold, full=args.full)
+            prev_sol_success = True
+        elif prev_sol_path.is_dir():
+            try:
+                prev_sol, i = get_recent_solution_and_iteration(args.prev_sol, args.startsol)
+            except:
+                warn('Could not find solution in directory %s, computing new starting solution' % args.prev_sol)
+            else:
+                a = a ** i
+                model = create_enum_variables(model, reaction_weights, eps=args.epsilon, thr=args.threshold,
+                                              full=args.full)
+                prev_sol_success = True
+        else:
+            warn('Could not read previous solution at path %s, computing new starting solution' % args.prev_sol)
+    if not prev_sol_success:
         prev_sol = imat(model, reaction_weights, epsilon=args.epsilon, threshold=args.threshold)
     icut = False if args.noicut else True
-    save = True if args.save else False
-    full = True if args.full else False
 
     dex_sol, dex_res = diversity_enum(model=model, reaction_weights=reaction_weights, prev_sol=prev_sol,
                                       thr=args.threshold, maxiter=args.maxiter, obj_tol=args.obj_tol, dist_anneal=a,
-                                      icut=icut, full=args.full, save=save)
+                                      out_path=args.output, icut=icut, full=args.full, save=args.save)
     dex_res.to_csv(args.output + '_results.csv')
-    dex_sol.to_csv(args.output + '_solutions.csv')
+    sol = pd.DataFrame(dex_sol.binary, columns=[r.id for r in model.columns])
+    sol.to_csv(args.output + '_solutions.csv')
+    return True
+
+
+if __name__ == '__main__':
+    main()
